/********************************************************
			BOOTLOADER HEADER FILE
	- All parameters must be set here to customize to 
	  actual hardware
***********************************************************/
/********************************************
		DEFINITIONS
	- Entry points for user app. 
********************************************/

	#define UserEntryAddress 0x0200
	#define UserHInt UserEntryAddress + 4	

/* Header Test Switches */

/************** Processor type selection *******************
 - P18f has been tested with 18f252 processor
 - p16f has been tested with 16f876, in this family
   processor must have a USART of course 
 **********************************************************/
// select one and only one of the 2 following lines

#pragma config |= 0x3F72


/********************************************
		CLOCKING PARAMETERS
 - Change processor clock here.
 - Support is at Khz here, no more decimal. 
********************************************/

// Clocking
#define CPUCLK					20	// in Mhz
#define CYCLK					CPUCLK / 4L	// Instruction cycle (cpu div by 4)

// process value of USART BRG VALUE, BRGH = 1
#define BAUDR			19200L			// value of baud rate needed
#define CONSTBR1		CYCLK * 1000000L
#define CONSTBR2		CONSTBR1 / 4L
#define CONSTBR3		CONSTBR2 / BAUDR
#define BAUDGEN			CONSTBR3 - 1L

#if (BAUDGEN) > (255L)
	#error Baud rate value not possible with BRDH=1  
#endif

/* **************** FLASH PROGRAM CAPACITY ***************/
// capacity of pgm memory in bytes (2 times number of 14bits words)
#define FLASHCAPACITY 256		// 16 k for 16f876

/********************************************
		GLOBAL VARIABLES
********************************************/
bit b_OK;
char rxdata, bi, b_tp;
#pragma rambank 1
char b_bufflash[64];
#pragma rambank 0


// dump values sent to pc in case or error
char dmpadrh,dmpadrl,dmpdathrd,dmpdatlrd,dmpdathwr,dmpdatlwr;


/********************************************
		HARDWARE DEFINITIONS
********************************************/
// Define the switch port that will trigger bootloading
// switch activation must be 0 if switch makes 0 when pressed
// otherwise must be 1
#pragma bit SWITCH @ PORTB.4
#define TrisSw TRISB.4
#define SwitchActivation 0

// If a led is available for status info, define it here
#define LED PORTB.1
#define TrisLed TRISB.1 = 0
#define LEDON 1;
#define LEDOFF 0;

	
/***********************************************
		PROTOTYPES
*************************************************/

void usermain (void);
void UserHighIntServ (void);
void UserLowIntServ (void);
void userreset (void);
