/**********************************************************
		RS232 SERIAL BOOTLOADER
 
 - All polled, no interrupt is used in this soft.
 - All parameters must be set in the corresponding .h file.
 
 ===> after compil copy resulting asm into bo.asm.
 	  then recompile in assembly after replacement 
 	  of CALL to user soft by a GOTO usermain 
************************************************************/

#include "int16Cxx.h"
#include "Srl_Bootldr.h"

/****** V E C T O R  R E M A P P I N G **************************/

  /* -----------------------------------------------------------------
            INTERRUPTS HANDLER                
 	- interrups not used for bootloader
 	- for live soft, a simple jump to user interrupt handler
 --------------------------------------------------------------------- */

#pragma origin 0x004


void hrdw_int_high( void)
{
#asm
	call UserHighIntServ		// call chgd in goto by compiler
#endasm
}

/*------------- End Interrupts services --------------*/

/********************************************************
     BOOTLOADER ROUTINES
 **********************************************************/
/*
No tempo routing needed, only provided if changes are made and use it.


void b_tempo (uns16 dureems)
{
	while (dureems != 0)
	{
	TMR1H=TIMER1H;
	TMR1L=TIMER1L;
	TMR1IF = 0;
	TMR1ON = 1;
 	while (TMR1IF == 0);
	TMR1ON = 0;
   	dureems = dureems-1;
	}
} 
*/

/* ********** FLASH WRITE P16F*********** */

void boot_writeflash (uns16 fadr) // 64 bytes at once in bufflash
 {
	char i,j,k,ad,pointr;
	uns16 svfadr;

	svfadr = fadr;
	b_OK = 1;
	if (fadr >= (UserEntryAddress))	// protect booloader zone
	{
		GIE = 0;		// should be already at 0, for standardizing rtg
		for (i = 0; i != 32; i++)
		{
			svfadr = fadr + i;
			EEADR = svfadr.low8;
			EEADRH = svfadr.high8;
			pointr = i * 2;
			b_bufflash[pointr+1] &= 0x3F;
			EEDATA = b_bufflash[pointr];
			EEDATH = b_bufflash[pointr+1];
			EEPGD = 1;
			WREN = 1;
			EECON2 = 0x55;
			EECON2 = 0xAA;
			WR = 1;
			nop();
			nop();
		// Next is according to a silicon errata from  Microchip 
		// see doc ds80053b 
#asm
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
#endasm
			WREN = 0;
			EEIF = 0;
		}
	}
	else b_OK = 1;

//	Check write
	if (fadr >= (UserEntryAddress))	// protect booloader zone
	{
		for (i = 0; i != 32; i++)
		{
			svfadr = fadr + i;
			EEADR = svfadr.low8;
			EEADRH = svfadr.high8;
			pointr = i * 2;		
			EEPGD = 1;
			RD = 1;
			nop();
			nop();
			j = EEDATA;
			k = EEDATH;
			if (j != b_bufflash[pointr])
				b_OK = 0;
			if (k != b_bufflash[pointr+1])
				b_OK = 0;
			EEIF = 0;
			if (b_OK == 0)
			{
				dmpadrh = svfadr.high8;
				dmpadrl = svfadr.low8;
				dmpdathrd = k;
				dmpdatlrd = j;
				dmpdathwr = b_bufflash[pointr+1];
				dmpdatlwr = b_bufflash[pointr];
			}	
		}
	}
}
	

//  *** serial port routines
void boot_outsrl (char outcar)
	{
	while (TRMT == 0);
	TXREG = outcar;
	}

char boot_waitsrl (void)
{
while (RCIF == 0)
	{nop();
	nop();
	}
return RCREG;
}

void bootprocess (void)
{
// *** main bootloader

	char rcvdata, i, j, BCC; 
	uns16 b_fadr,btloop;
	GIE = 0;
	char tp;
	bit OutRng;
	char kflash;

/* host synch*/
	bi = 0;
	OutRng = 0;
	b_tp=0;
	kflash = FLASHCAPACITY / 2;
	while (bi == 0)
	{
				rxdata = boot_waitsrl();
		if (rxdata == 'R')
		{
			nop();
			nop();
			rxdata = boot_waitsrl();

			if (rxdata == 'M')
			{
				nop();
				nop();
				rxdata = boot_waitsrl();
				if (rxdata == 'b')
				{
					nop();
					nop();
					rxdata = boot_waitsrl();
					if (rxdata == 't')
					{
#ifdef LED						
						LED = LEDOFF;
#endif
						boot_outsrl ('b');			// Device reply with c to complete handshake
						boot_outsrl ('t');
						boot_outsrl (FLASHCAPACITY);
						bi = 1;
					}	
				}
			}
		} 
	}

/* tfr data */
	b_fadr = 0;

// transfer format is :
// 27 as sart of block
// address high, address low, in hex.
// then 64 bytes follows, then check caracter
// When address is 0xff00: stop

	while (b_fadr != 0xFF00)	// must complete with address FF00 to stop tfr
	{
		while (boot_waitsrl() != 27);		// start of block 
		rcvdata = boot_waitsrl();
		BCC = rcvdata;
		b_fadr.high8 = rcvdata;
		rcvdata = boot_waitsrl();
		BCC += rcvdata;
		b_fadr.low8 = rcvdata;
		if (b_fadr.high8 >= kflash)
		{
			OutRng = 1;
		}	
		if (b_fadr == 0xFF00) break;
		for (i = 0;i!=64;i++)
		{
			rcvdata = boot_waitsrl();
			BCC += rcvdata;
			b_bufflash[i] = rcvdata;
		}
#ifdef LED
		b_tp++;
		if (b_tp == 5)
		{
			LED = !LED;
			b_tp=0;
		}
#endif	
		rcvdata = boot_waitsrl();	// BCC caracter
		if (rcvdata == BCC)  
		{
			b_OK=1;
			if (OutRng == 0)
			{		
				boot_writeflash (b_fadr);
				if (b_OK == 1)
				{
					boot_outsrl (6);		// check is ok, reply ack
				}
				else
				{
					boot_outsrl (8);
					boot_outsrl (dmpadrh);
					boot_outsrl(dmpadrl);
					boot_outsrl(dmpdathrd);
					boot_outsrl(dmpdatlrd);
					boot_outsrl(dmpdathwr);
					boot_outsrl(dmpdatlwr);
				}
			}
			else 
			{
				boot_outsrl(9);
				b_fadr = 0xFF00;
			}			
	
		}	
		else
		{
				boot_outsrl (7);	// reply nack
		}
	}
}

void Initialize()
{
	INTCON = 0;		//  disable all interrupts /

	STATUS = 0;
	PORTA = 0; 
	PORTB = 0;
	PORTC = 0;  
	TRISA = bin(111111);
	TRISB = bin(11111111);
	TRISC = bin(11111111);

#ifdef LED
	TrisLed;
#endif
	/* init periph interrupts */
	PIE1 = bin(00000000); /* disabled int adc, usart, all polled */
	PIR1 = 0;
	PIE2 = bin(00000000);	// int to clear eeprom wr 
	PIR2 = 0;
	GIE = 0;			// no interrupt, polling hrdw
	clearRAM();
	/* init USART 19.2 Kbps / 8 bits*/
	TXSTA = 0b00100100;
	RCSTA = 0b10010000;
//	BRGH = 0;
	TRISC.6 = 1;
	TRISC.7 = 1;
	SPBRG = (char)BAUDGEN;		//   ****** verif params in .h *******
	bi = RCREG;	// dummy reads to clear buffer.
	nop();
	bi = RCREG;
}	


/************* RESET ENTRY POINT ***************/
void main( void)
{
	ADCON0 = 0;		// All analog ports as digital
	ADCON1 = 0x07;
// next line not needed, just in case wrong manip made on previous lines
	TrisSw = 1;		// input of course	
// test if switch pressed, otherwise jump to user code
	if (SWITCH == SwitchActivation)
	{
		Initialize();
#ifdef LED
	LED = LEDON;
#endif
		bootprocess();
	}
// if switch not pressed goto user startup

// CC5X does not allow a goto here below
// only way to change to GOTO is to recompile in assembly later

	#asm
		CALL usermain		// call should be chgd in goto in asm
	#endasm

}

/* ---------------------------------------------------------
              DUMMY MAIN        
 	- This part is only here to generate the necessary
 	  adresses for the direct calls                           
 ---------------------------------------------------------*/

#pragma origin UserEntryAddress
void usermain( void)
{
// user code should start like the following code :
#asm
	call userreset
#endasm
}

#pragma origin UserHInt

void UserHighIntServ (void)
{
}


//  small example of structure of user code :	
void MainLoop (void)
{
}

void SystemInit (void)
{
}		
void userreset ( void)
{
 	SystemInit();
	while (1)
	{
		TRISB.2 = 0;
		PORTB.2 = 1;
		MainLoop();
	}	
}
