
/* ------------------------------------------------*/
/*             BOOT LOADER                         */
/*-------------------------------------------------*/

#pragma codepage 0
// whole page 0 is reserved for boot loader


/* ********** FLASH WRITE ************* */

void b_writeflash (uns16 fadr) // 64 bytes at once in bufflash
 {
	char i,j,k,ad,pointr;
	uns16 svfadr;

	svfadr = fadr;
	b_OK = 1;
	if (fadr < (0x1780))	// protect booloader zone
	{
		GIE = 0;		// should be already at 0, for standardizing rtg
		for (i = 0; i != 32; i++)
		{
			svfadr = fadr + i;
			EEADR = svfadr.low8;
			EEADRH = svfadr.high8;
			pointr = i * 2;
			b_bufflash[pointr+1] &= 0x3F;
			EEDATA = b_bufflash[pointr];
			EEDATH = b_bufflash[pointr+1];
			EEPGD = 1;
			WREN = 1;
			EECON2 = 0x55;
			EECON2 = 0xAA;
			WR = 1;
			nop();
			nop();
		// Next is according to a silicon errata from  Microchip 
		// see doc ds80053b 
#asm
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
	DW 0x3FFF
#endasm
			WREN = 0;
			EEIF = 0;
		}
	}
else b_OK = 1;

//	Check write
		for (i = 0; i != 32; i++)
		{
			svfadr = fadr + i;
			EEADR = svfadr.low8;
			EEADRH = svfadr.high8;
			pointr = i * 2;		
			EEPGD = 1;
			RD = 1;
			nop();
			nop();
			j = EEDATA;
			k = EEDATH;
			if (j != b_bufflash[pointr])
				b_OK = 0;
			if (k != b_bufflash[pointr+1])
				b_OK = 0;
			EEIF = 0;
			if (b_OK == 0)
			{
				dmpadrh = svfadr.high8;
				dmpadrl = svfadr.low8;
				dmpdathrd = k;
				dmpdatlrd = j;
				dmpdathwr = b_bufflash[pointr+1];
				dmpdatlwr = b_bufflash[pointr];
			}	
		}

}	

//  *** serial port routines
void b_outsrl (char outcar)
	{
	while (TRMT == 0);
	TXREG = outcar;
	}

char b_waitsrl (void)
{
while (RCIF == 0)
	{nop();
	nop();
	}
return RCREG;
}

// *** main bootloader
void bootloader (void)
{
	char rcvdata, i, j, BCC; 
	uns16 b_fadr,btloop;
	b_resetinit();
	GIE = 0;
	char tp;

/* host synch*/
	bi = 0;
	b_tp=0;
	while (bi == 0)
	{
		rxdata = b_waitsrl();
		if (rxdata == 'C')
		{
			nop();
			nop();
			rxdata = b_waitsrl();
			if (rxdata == 'D')
			LED = 1;
			{
				nop();
				nop();
				b_outsrl ('c');			// Device reply with c to complete handshake
				bi = 1;
			}
		} 
	}

/* tfr data */
	b_fadr = 0;

// transfer format is :
// 27 as sart of block
// address high, address low, in hex.
// then 64 bytes follows, then check caracter
// When address is 0xff00: stop

	while (b_fadr != 0xFF00)	// must complete with address FF00 to stop tfr
	{
		while (b_waitsrl() != 27);		// start of block 
		rcvdata = b_waitsrl();
		BCC = rcvdata;
		b_fadr.high8 = rcvdata;
		rcvdata = b_waitsrl();
		BCC += rcvdata;
		b_fadr.low8 = rcvdata;
		for (i = 0;i!=64;i++)
		{
			rcvdata = b_waitsrl();
			BCC += rcvdata;
			b_bufflash[i] = rcvdata;
		}
		b_tp++;
		if (b_tp == 5)
		{
			LED = !LED;
			b_tp=0;
		}	
		rcvdata = b_waitsrl();	// BCC caracter
		if (rcvdata == BCC)  
		{
			b_OK=1;		
				b_writeflash (b_fadr);
		for (btloop = 0;btloop!=500;btloop++);
				if (b_OK == 1)
				{
					b_outsrl (6);		// check is ok, reply ack
				}
				else
				{
					b_outsrl (8);
					b_outsrl (dmpadrh);
					b_outsrl(dmpadrl);
					b_outsrl(dmpdathrd);
					b_outsrl(dmpdatlrd);
					b_outsrl(dmpdathwr);
					b_outsrl(dmpdatlwr);
				}
	
		}
		else
		{
				b_outsrl (7);	// reply nack
		}
	}
	while (1);
}

