#ifndef SrlBoot
#define SrlBoot

/********************************************************
			BOOTLOADER HEADER FILE
	- All parameters must be set here to customize to 
	  actual hardware
***********************************************************/
	

/* Header Test Switches */

/************** Processor type selection *******************
 - P18f has been tested with 18f252 processor and 18f4550
 
 **********************************************************/
#define Quartz24Mhz
/* optionaly config may be set here 
 config must be used with care here check with exact proc type
*/

#if defined(_18F2550) || (_18F4550)
#if defined Quartz20Mhz 
#pragma config [0]  = 0x24
#endif
#if defined Quartz24Mhz
#pragma config [0]  = 0x25
#endif
#if defined Quartz16Mhz
#pragma config [0]  = 0x23
#endif
#pragma config [1]  = 0x0E
#pragma config [2]  = 0b00111111
#pragma config [3]  = 0b00011110
#pragma config [4]  = 0
#pragma config [5]  = 0b10000001
#pragma config [6]  = 0b10000001
#pragma config [7]  = 0
#pragma config [8]  = 0b00001111
#pragma config [9]  = 0b11000000
#pragma config [10] = 0b00001111
#pragma config [11] = 0b11100000
#pragma config [12] = 0b00001111
#pragma config [13] = 0b01000000
#endif
#if defined(_18F252)
#pragma config [0]  = 0
#pragma config [1]  = 0b00100101
#pragma config [2]  = 0b00001110
#pragma config [3]  = 0b00001110
#pragma config [4]  = 0
#pragma config [5]  = 0b00000001
#pragma config [6]  = 0b10000001
#pragma config [7]  = 0
#pragma config [8]  = 0b00001111
#pragma config [9]  = 0b11000000
#pragma config [10] = 0b00001111
#pragma config [11] = 0b11100000
#pragma config [12] = 0b00001111
#pragma config [13] = 0b01000000
#endif
#if defined(_18F25K50) || (_18F45K50)
//  CONFIG bits  >> clock intr 16mhz pll x 3
#pragma config [0]  = 0x23	//LS48MHZ=1(48mhz sys clk), CPUDIV =00 (no),CFGPLLEN=0 (oscillator direct),PLLSEL=1(pll x 3)
#pragma config [1]  = 0x08	//IESO=0(no osc switchover),FCMEN=0(no clk failsafe),PCLKEN=0(primary clk can be dis),FOSC=1000(internal)
#pragma config [2]  = 0x0A	//LPBOR=0(BRO rst enab),BORV=01(2.5v),BOREN=01(bro crtrld by soft),PWRTEN=0(pwr up tmr ena)
#pragma config [3]  = 0x3E	//WDTPS=1111(max32768),WDTEN=10(soft ctrld)
#pragma config [4]  = 0		//NU
#pragma config [5]  = 0x91	//MCLRE=1(ext reset),SDOMX=0(SDO on RC7),T3CM=1(T3CKI on RC0),PBADEN=0(ANSELB),CCP2MX=1(CCP2 on RC1)
#pragma config [6]  = 0x81	//DEBUG=1(disable),XINST=0(extended instr dis),ICPRT=0(ICPORT dis),LPV=0(low volt ICSP dis),STRVEN=1(stack ovflow reset)
#pragma config [7]  = 0		//NU
// next part is code protection. No protection used.
#pragma config [8]  = 0x0F	//
#pragma config [9]  = 0xC0	//
#pragma config [10] = 0x0F	//
#pragma config [11] = 0xE0	//
#pragma config [12] = 0x0F	//
#pragma config [13] = 0x40	//
#endif
/****************************************
		FLASH CAPACITY
****************************************/
#define FLASHCAPACITY 32	//in KBytes

/********************************************
		CLOCKING and OTHER CHIP PARAMETERS
 - Change processor clock here.
 - Clock in MHz, e.g. 20 for 20Mhz quartz (no pll)
********************************************/

// Clocking
#define CPUCLK					20		// in Mhz
#define CYCLK					CPUCLK / 4	// Instruction cycle (cpu div by 4)
//#define 28Pins
//#define 40Pins
// process value of USART BRG VALUE, BRGH = 1
#define BAUDR			19200L			// value of baud rate needed
#define CONSTBR1		CYCLK * 1000000 	//in Hz
#define CONSTBR2		CONSTBR1 / 4
#define CONSTBR3		CONSTBR2 / BAUDR
#define BAUDGEN			CONSTBR3 - 1

#if (BAUDGEN) > (255L)
	#error Baud rate value not possible with BRDH=1  
#endif


/********************************************
		HARDWARE DEFINITIONS
********************************************/
// Define the switch port that will trigger bootloading
// switch activation must be 0 if switch makes 0 when pressed
// otherwise must be 1, here portb bit 4
#define SWITCH PORTB.4
#define TrisSw TRISB.4 
#define SwitchActivation 0

// If a led is available for status info, define it here

#define LED  PORTB.1
#define TrisLed TRISB.1 = 0;	// led tris pins	
#define LEDON 1
#define LEDOFF 0
 
/********************************************
		DEFINITIONS
	- Entry points for user app. 
	Can be 0x0400, or 0x1000 for compatibilty with USB BootLoader
********************************************/
#define UserEntryAddress 0x1000
#define UserHInt UserEntryAddress + 8
#define UserLInt UserEntryAddress + 0x18

/********************************************
		GLOBAL VARIABLES
********************************************/
bit boot_OK;
char rxdata, bi, boot_tp;
#pragma rambank 1
char boot_bufflash[64];
#pragma rambank 0

// dump values sent to pc in case or error
char dmpadrh,dmpadrl,dmpdathrd,dmpdatlrd,dmpdathwr,dmpdatlwr;

/***********************************************
		PROTOTYPES
*************************************************/

void usermain (void);
void UserHighIntServ (void);
void UserLowIntServ (void);
void userreset (void);

#endif
